module ahb_fpga_sram
  #(
    parameter AW       = 12 // Address width
    )
   (
    input wire 		HCLK, // Clock
    input wire 		HRESETn, // Reset
    // AHB inputs
    input wire 		HSEL, // Device select
    input wire [AW-1:0] HADDR, // Address
    input wire [1:0] 	HTRANS, // Transfer control
    input wire [2:0] 	HSIZE, // Transfer size
    input wire 		HWRITE, // Write control
    input wire [31:0] 	HWDATA, // Write data
    input wire 		HREADY, // Transfer phase done
    // AHB Outputs
    output wire 	HREADYOUT, // Device ready
    output wire [31:0] 	HRDATA, // Read data output
    output wire 	HRESP
    );  // Device response (always OKAY)

   wire [AW-3:0] 	SRAMADDR;
   wire [31:0] 		SRAMWDATA;
   wire [31:0] 		SRAMRDATA;
   wire [3:0] 		SRAMWEN;
   wire 		SRAMCS;

   // AHB to SRAM bridge
   cmsdk_ahb_to_sram #(.AW(AW)) u_ahb_to_sram
     (
      // AHB Inputs
      .HCLK       (HCLK),
      .HRESETn    (HRESETn),
      .HSEL       (HSEL),  // AHB inputs
      .HADDR      (HADDR),
      .HTRANS     (HTRANS),
      .HSIZE      (HSIZE),
      .HWRITE     (HWRITE),
      .HWDATA     (HWDATA),
      .HREADY     (HREADY),
      
      // AHB Outputs
      .HREADYOUT  (HREADYOUT), // Outputs
      .HRDATA     (HRDATA),
      .HRESP      (HRESP),
      
      // SRAM input
      .SRAMRDATA  (SRAMRDATA),
      // SRAM Outputs
      .SRAMADDR   (SRAMADDR),
      .SRAMWDATA  (SRAMWDATA),
      .SRAMWEN    (SRAMWEN),
      .SRAMCS     (SRAMCS)
      );
   
   // SRAM model
   cmsdk_fpga_sram #(.AW(AW)) u_fpga_sram
     (
      // SRAM Inputs
      .CLK        (HCLK),
      .ADDR       (SRAMADDR),
      .WDATA      (SRAMWDATA),
      .WREN       (SRAMWEN),
      .CS         (SRAMCS),
      // SRAM Outputs
      .RDATA      (SRAMRDATA)
      );

endmodule // ahb_fpga_sram
