`include "vdefs.v"

module comp
  (
   input wire 	      CLK,
   input wire 	      RESETn,
   input wire 	      BOOT_MODE,
   input wire 	      CLK_1M,
 
   output wire [31:0] PA_P0,
   output wire [31:0] PB_P0,
   output wire [31:0] PC_P0,
   output wire [31:0] PD_P0,
   input wire [31:0]  PI_P0,
   input wire [31:0]  PJ_P0,
   
   input wire 	      RXD0_P0,
   output wire 	      TXD0_P0,
   input wire 	      RXD1_P0,
   output wire 	      TXD1_P0,
   input wire 	      RXD2_P0,
   output wire 	      TXD2_P0,
   output wire 	      BAUDTICK_P0,
   input wire 	      TIN_P0,
   
   // test signals
   output wire [31:0] TST_HADDR
   );
   
   // AHB signals from P0 and P1, connected to BLD.A and BLD.B
   wire 	       HCLK_P0;
   wire 	       HSEL_P0_BLD;
   wire [31:0] 	       HADDR_P0;
   wire [1:0] 	       HTRANS_P0;
   wire 	       HREADY_P0;
   wire [31:0] 	       HWDATA_P0;
   wire 	       HWRITE_P0;
   wire [2:0] 	       HSIZE_P0;
   wire [2:0] 	       HBURST_P0;
   wire [3:0] 	       HPROT_P0;
 
   // Signals of BLD back to Px
   wire [31:0] 	       HRDATA_P0_BLD;
   wire 	       HREADYOUT_P0_BLD;
   wire 	       HRESP_P0_BLD;

   // Port wires

   // Clock
   wire 	       CLK_P0;
   assign CLK_P0= CLK;

   // Reset
   reg [3:0] reset_queue= 4'd0;
   always @(posedge CLK_P0)
     reset_queue<= {reset_queue[2:0],RESETn};
   wire 	       HRESETn_P0;
   assign HRESETn_P0= &(reset_queue);

   // MCU: CPU+Mem+Periphs
   mcu0
     #(
       .CODE_AW(`CODE_AW),
       .DATA_AW(`DATA_AW),
       .CODE_file("brgy")
       )
   P0
     (
      .CLK		(CLK_P0),
      .RESETn		(HRESETn_P0),
      .BOOT_MODE	(BOOT_MODE),
      .SWCLKTCK		(1'b0),
      .SWDITMS		(),
      .SWDO		(),
      .SWDOEN		(),
      .CLK_1M		(CLK_1M),
      .RXEV		(1'b1),
      .TXEV		(),
      
      .PA		(PA_P0),
      .PB		(PB_P0),
      .PC		(PC_P0),
      .PD		(PD_P0),
      .PI		(PI_P0),
      .PJ		(PJ_P0),
      .RXD0		(RXD0_P0),
      .TXD0		(TXD0_P0),
      .BAUDTICK		(BAUDTICK_P0),
      .TIN      (TIN_P0)
      );

assign TST_HADDR= HADDR_P0;

endmodule // comp
