`include "vdefs.v"

module ahb_mrom
  #(
    parameter MEMFILE= "",
    parameter AW= 16
    )
   (
    input wire 	      HCLK_A,
    input wire 	      HSEL_A,
    input wire [31:0] HADDR_A,
    input wire [1:0]  HTRANS_A,
    input 	      HREADY_A,
    output reg [31:0] HRDATA_A,
    output 	      HREADYOUT_A,
    output 	      HRESP_A,

    input wire 	      HCLK_B,
    input wire 	      HSEL_B,
    input wire [31:0] HADDR_B,
    input wire [1:0]  HTRANS_B,
    input 	      HREADY_B,
    output reg [31:0] HRDATA_B,
    output 	      HREADYOUT_B,
    output 	      HRESP_B
);

   localparam MEMDEPTH = 1 << AW;

   //Wire/Reg declarations
   reg [31:0] 	      rom [0:MEMDEPTH-1];

   wire 	      trans_valid_a = HSEL_A & HTRANS_A[1] & HREADY_A;
   wire 	      trans_valid_b = HSEL_B & HTRANS_B[1] & HREADY_B;

   wire [AW-1:0] rom_index_a = HADDR_A[AW+1:2];
   wire [AW-1:0] rom_index_b = HADDR_B[AW+1:2];
    
   always @(posedge HCLK_A)
     if (trans_valid_a)
       HRDATA_A <= rom[rom_index_a];

   always @(posedge HCLK_B)
     if (trans_valid_b)
       HRDATA_B <= rom[rom_index_b];

   assign HREADYOUT_A = 1'b1;
   assign HREADYOUT_B = 1'b1;
   
   assign HRESP_A     = 1'b0;
   assign HRESP_B     = 1'b0;

   reg [128*8:1] string;
   initial
     begin
	if (MEMFILE != "")
	  string= { `SW_PATH,"/",MEMFILE };
	  $display("Loading ahb_mrom from %s into %m", string);
	  $readmemh(string, rom);
     end
   
endmodule // ahb_rom
