module ahb_watch_v2
  #(
    parameter ADDRWIDTH= 12
    )
  (
   input wire 		      HCLK, // Clock
   input wire 		      HRESETn, // Reset
    
   input wire [3:0] 	      ECOREVNUM, // Engineering-change-order revision bits
    
    // AHB connection to master
   input wire 		      HSEL,
   input wire [ADDRWIDTH-1:0] HADDR,
   input wire [1:0] 	      HTRANS,
   input wire [2:0] 	      HSIZE,
   input wire 		      HWRITE,
   input wire 		      HREADY,
   input wire [31:0] 	      HWDATA,
    
   output wire 		      HREADYOUT,
   output wire 		      HRESP,
   output wire [31:0] 	      HRDATA
   );

   
   // ----------------------------------------
   // Internal wires declarations
   
   // Register module interface signals
   wire [ADDRWIDTH-1:0]        reg_addr;
   wire 		       reg_read_en;
   wire 		       reg_write_en;
   wire [3:0] 		       reg_byte_strobe;
   wire [31:0] 		       reg_wdata;
   wire [31:0] 		       reg_rdata;

   cmsdk_ahb_eg_slave_interface
     #(.ADDRWIDTH (ADDRWIDTH))
   watch_ahb_interface
     (
      .hclk         (HCLK),
      .hresetn      (HRESETn),
      
      // Input slave port: 32 bit data bus interface
      .hsels        (HSEL),
      .haddrs       (HADDR),
      .htranss      (HTRANS),
      .hsizes       (HSIZE),
      .hwrites      (HWRITE),
      .hreadys      (HREADY),
      .hwdatas      (HWDATA),
      
      .hreadyouts   (HREADYOUT),
      .hresps       (HRESP),
      .hrdatas      (HRDATA),
      
      // Register interface
      .addr         (reg_addr),
      .read_en      (reg_read_en),
      .write_en     (reg_write_en),
      .byte_strobe  (reg_byte_strobe),
      .wdata        (reg_wdata),
      .rdata        (reg_rdata)
      );

   watch_v2 the_watch
     (
      .hclk		(HCLK),
      .hresetn		(HRESETn),
      .addr		(reg_addr),
      .read_en		(reg_read_en),
      .write_en		(reg_write_en),
      .byte_strobe	(reg_byte_strobe),
      .wdata		(reg_wdata),
      .ecorevnum	(ECOREVNUM),
      .rdata		(reg_rdata)
      );

   
endmodule // ahb_watch_v2
