//-----------------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 2008-2010 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//
//      SVN Information
//
//      Checked In          : $Date: 2008-12-08 12:21:19 +0000 (Mon, 08 Dec 2008) $
//
//      Revision            : $Revision: 96209 $
//
//      Release Information : Cortex-M0-AT510-r0p0-03rel0
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//     MCU System level for Cortex-M0 Integration Kit
//-----------------------------------------------------------------------------
// 
// This level instantiates the following blocks:
//  o u_sys            - Integration Kit System Level
//  o u_pmu               - Cortex-M0 Power Management Unit
//  o u_clk_gen           - Clock Generator
//  o u_rst_ctl           - Reset Controller
//
// Additionally this level contains tristates on bi-directional pins from MCU
//-----------------------------------------------------------------------------

module dd_cpu
  (
   input wire 	      CLK, // Clock
   input wire 	      PORESETn, // Power-On-Reset
   input wire 	      SWCLKTCK, // Serial Wire Clock or TCK
   input 	      SWDITMS,
   output 	      SWDO,
   output 	      SWDOEN, 
   input wire 	      TDI, // Debug TDI
   output wire 	      TDO, // Debug TDO
   // Signals by dd
   input wire 	      RXEV,
   output wire 	      TXEV,
   input wire [31:0]  IRQ,
     // AHB signals go outside
   output wire [31:0] HADDR,
   output wire [31:0] HWDATA,
   output wire 	      HWRITE,
   output wire [2:0]  HSIZE,
   output wire [2:0]  HBURST,
   output wire [3:0]  HPROT,
   output wire [1:0]  HTRANS,
   // Answers from outsider-system
   input wire [31:0]  oHRDATA,
   input wire 	      oHREADY,
   input wire 	      oHRESP
   );
   
// --------------------------------------------------------------------------
// Wires & Regs
// --------------------------------------------------------------------------
   wire 	      mcu_fclk;              // Free-running clock
   wire 	      mcu_sclk;              // System clock   
   wire 	      mcu_hclk;              // AHB clock
   wire 	      mcu_dclk;              // Debug clock   
   wire 	      mcu_stclk;             // Systick clock
   wire 	      mcu_wicenreq;          // WIC-enable Requrest   
   wire 	      mcu_wicenack;          // WIC enable Ack   
   wire 	      mcu_cdbgpwrupreq;      // Debug Power Up Request   
   wire 	      mcu_cdbgpwrupack;      // Debug Power Up Ack   
   wire 	      mcu_sleepholdreqn;     // Extended sleep Request   
   wire 	      mcu_sleepholdackn;     // Extended sleep Ack
   wire 	      mcu_sysisolaten;       // SRPG System Isolate
   wire 	      mcu_sysretainn;        // SRPG System Retain 
   wire 	      mcu_syspwrdown;        // SRPG System Power Down
   wire 	      mcu_dbgisolaten;       // SRPG Debug Isolate
   wire 	      mcu_dbgpwrdown;        // SRPG Debug Power Down
   wire 	      mcu_wakeup;            // WIC Wake up
   wire 	      mcu_sleepdeep;         // Sleepdeep from CM0 
   wire 	      mcu_gatehclk;          // Gate HCLK  
   wire 	      mcu_hresetreq;         // HRESET Request   
   wire 	      mcu_sysresetreq;       // SYSRESET Request 
   wire 	      mcu_pmuhresetreq;      // PMU HRESET Requeset
   wire 	      mcu_pmudbgresetreq;    // PMU DBGRESET Request
   wire 	      mcu_poresetn;          // Power-On Reset from Reset Controller
   wire 	      mcu_hresetn;           // System Reset from Reset Controller
   wire 	      mcu_dbgresetn;         // Debug Reset from Reset Controller
   wire 	      mcu_syspwrdownack;     // SYSPWRDOWNACK from CM0   
   wire 	      mcu_dbgpwrdownack;     // DBGPWRDOWNACK from CM0
   wire 	      pmu_syspwrdownack;     // SYSPWRDOWNACK to PMU   
   wire 	      pmu_dbgpwrdownack;     // DBGPWRDOWNACK to PMU   
   wire 	      mcu_swdo;              // SW data out 
   wire 	      mcu_swdoen;            // SW data out enable
   wire 	      mcu_ntdoen;            // JTAG TDO enable
   wire 	      mcu_swditms;           // SW data in
   wire 	      mcu_tdo;               // TDO
   wire               sys_stclken;
   //wire [31:0] mcu_extgpioin;         // GPIOIN from IKSYS
   //wire [31:0] mcu_extgpioout;        // GPIOOUT from IKSYS  
   //wire [31:0] mcu_extgpioen;         // GPIOEN from IKSYS
   integer 	      i;

`ifdef ARM_SRPG_ON
`ifdef ARM_UPF_INVERT_PWRDOWNACK
   // Invert PWRDOWNACK for UPF
   assign    pmu_syspwrdownack = ~mcu_syspwrdownack;
   assign    pmu_dbgpwrdownack = ~mcu_dbgpwrdownack;
`elsif ARM_CPF_PWRDOWNACK
   // PWRDOWACK not driven for CPF
   assign    pmu_syspwrdownack = mcu_syspwrdown;
   assign    pmu_dbgpwrdownack = mcu_dbgpwrdown;
`else
   // For everything else including Netlists
   assign    pmu_syspwrdownack = mcu_syspwrdownack;
   assign    pmu_dbgpwrdownack = mcu_dbgpwrdownack;
`endif 
`else
   // Non Power-aware simulation
   assign    pmu_syspwrdownack = mcu_syspwrdown;
   assign    pmu_dbgpwrdownack = mcu_dbgpwrdown;
`endif // ARM_SRPG_ON
   
   
   //-----------------------------------------------------------------------------
   // STCLKEN Generation
   //-----------------------------------------------------------------------------
   cm0ik_stclken_gen  u_stclken
     (//Outputs
      .STCLKEN                           (sys_stclken),
      //Inputs
      .SCLK                              (mcu_sclk),
      .HRESETn                           (mcu_hresetn),
      .STCLK                             (mcu_stclk)
      );
   
   
   dd_cm0
   //cm0ik_cortexm0_timing
     #(.CAPTUREVEC                        (1))
   //u_timing
   u_cm0
     (// Inputs
      .PORESETn                          (mcu_poresetn),
      .DBGRESETn                         (mcu_dbgresetn),
      .HRESETn                           (mcu_hresetn),
      .FCLK                              (mcu_fclk),
      .SCLK                              (mcu_sclk),
      .HCLK                              (mcu_hclk),
      .DCLK                              (mcu_dclk),
      .DBGRESTART                        (1'b0/*sys_dbgrestart*/),
      .NMI                               (1'b0/*sys_nmi*/),
      .IRQ                               (IRQ),
      .RXEV                              (RXEV),
      .SE                                (1'b0),
      .RSTBYPASS                         (1'b0),
      // Outputs
      .HMASTER                           (),
      .DBGRESTARTED                      (/*sys_dbgrestarted*/),
      .HALTED                            (/*sys_halted*/),
      .TXEV                              (TXEV),
      .LOCKUP                            (/*sys_lockup*/),
      .CODENSEQ                          (),
      .CODEHINTDE                        (),
      .SPECHTRANS                        (),
      // Outputs
      .HADDR                             (HADDR),
      .HBURST                            (HBURST),
      .HMASTLOCK                         (/*sys_hmastlock_cortexm0*/),
      .HPROT                             (HPROT),
      .HSIZE                             (HSIZE),
      .HTRANS                            (HTRANS),
      .HWDATA                            (HWDATA),
      .HWRITE                            (HWRITE),
      .SWDO                              (SWDO),
      .SWDOEN                            (SWDOEN),
      .TDO                               (mcu_tdo),
      .nTDOEN                            (mcu_ntdoen),
      .SYSRESETREQ                       (mcu_sysresetreq),
      .GATEHCLK                          (mcu_gatehclk),
      .SLEEPING                          (/*sys_sleeping*/),
      .SLEEPDEEP                         (mcu_sleepdeep),
      .WAKEUP                            (mcu_wakeup),
      .SLEEPHOLDACKn                     (mcu_sleepholdackn),
      .WICENACK                          (mcu_wicenack),
      .WICSENSE                          (),
      .CDBGPWRUPREQ                      (mcu_cdbgpwrupreq),
      .SYSPWRDOWNACK                     (mcu_syspwrdownack),
      .DBGPWRDOWNACK                     (mcu_dbgpwrdownack),
      // Inputs
      .SWCLKTCK                          (SWCLKTCK),
      .nTRST                             (1'b1),
      .HRDATA                            (oHRDATA),
      .HREADY                            (oHREADY),
      .HRESP                             (oHRESP),
      .SWDITMS                           (SWDITMS),
      .TDI                               (TDI),
      .EDBGRQ                            (1'b0/*sys_edbgrq*/),
      .STCLKEN                           (sys_stclken),
      .SLEEPHOLDREQn                     (mcu_sleepholdreqn),
      .WICENREQ                          (mcu_wicenreq),
      .CDBGPWRUPACK                      (mcu_cdbgpwrupack),
      .SYSRETAINn                        (mcu_sysretainn),
      .SYSISOLATEn                       (mcu_sysisolaten),
      .SYSPWRDOWN                        (mcu_syspwrdown),
      .DBGISOLATEn                       (mcu_dbgisolaten),
      .DBGPWRDOWN                        (mcu_dbgpwrdown)
      );


//-----------------------------------------------------------------------------
// PMU
//-----------------------------------------------------------------------------

   cortexm0_pmu u_pmu
     (// Inputs
      .FCLK                            (mcu_fclk),
      .PORESETn                        (mcu_poresetn),
      .HRESETREQ                       (mcu_hresetreq),
      // Outputs
      .SCLK                            (mcu_sclk),
      .HCLK                            (mcu_hclk),
      .DCLK                            (mcu_dclk),
      .PMUHRESETREQ                    (mcu_pmuhresetreq),
      .PMUDBGRESETREQ                  (mcu_pmudbgresetreq),
      .WICENREQ                        (mcu_wicenreq),
      .CDBGPWRUPACK                    (mcu_cdbgpwrupack),
      .SLEEPHOLDREQn                   (mcu_sleepholdreqn),
      .SYSISOLATEn                     (mcu_sysisolaten),
      .SYSRETAINn                      (mcu_sysretainn),
      .SYSPWRDOWN                      (mcu_syspwrdown),
      .DBGISOLATEn                     (mcu_dbgisolaten),
      .DBGPWRDOWN                      (mcu_dbgpwrdown),
      // Inputs
      .PMUENABLE                       (1'b1),
      .CDBGPWRUPREQ                    (mcu_cdbgpwrupreq),
      .WICENACK                        (mcu_wicenack),
      .WAKEUP                          (mcu_wakeup),
      .SLEEPDEEP                       (mcu_sleepdeep),
      .SLEEPHOLDACKn                   (mcu_sleepholdackn),
      .GATEHCLK                        (mcu_gatehclk),
      .SYSPWRDOWNACK                   (pmu_syspwrdownack),
      .DBGPWRDOWNACK                   (pmu_dbgpwrdownack),
      .CGBYPASS                        (1'b0)
      );
   
//-----------------------------------------------------------------------------
// Clock Generator
//-----------------------------------------------------------------------------

   cm0ik_clk_gen u_clk_gen
     (//Inputs
      .CLK                             (CLK),
      .RESETn                          (mcu_poresetn),
      //Outputs
      .FCLK                            (mcu_fclk),
      .STCLK                           (mcu_stclk)
      );

 
//-----------------------------------------------------------------------------
// Reset Controller
//-----------------------------------------------------------------------------

   cortexm0_rst_ctl u_rst_ctl
     (// Inputs
      .GLOBALRESETn                    (PORESETn),
      .FCLK                            (mcu_fclk),
      .HCLK                            (mcu_hclk),
      .DCLK                            (mcu_dclk),
      .SYSRESETREQ                     (mcu_sysresetreq),
      .PMUHRESETREQ                    (mcu_pmuhresetreq),
      .PMUDBGRESETREQ                  (mcu_pmudbgresetreq),
      .RSTBYPASS                       (1'b0),
      .SE                              (1'b0),
      // Outputs
      .PORESETn                        (mcu_poresetn),
      .HRESETn                         (mcu_hresetn),
      .DBGRESETn                       (mcu_dbgresetn),
      .HRESETREQ                       (mcu_hresetreq)
      );

//-----------------------------------------------------------------------------
// Tristates on SWDIOTMS and TDO
//-----------------------------------------------------------------------------

   //assign mcu_swditms = SWDIOTMS;

   //bufif1 (SWDIOTMS, mcu_swdo, mcu_swdoen);
   bufif0 (TDO,      mcu_tdo,  mcu_ntdoen);
   
endmodule
