module mcu0
  #(
    parameter CODE_AW=12,
    parameter DATA_AW=12,
    parameter CODE_file=""
    )
   (
    input wire 	       CLK, // Clock
    input wire 	       RESETn, // Power-On-Reset
    input wire 	       BOOT_MODE,
    input wire 	       SWCLKTCK, // Serial Wire Clock or TCK
    input wire 	       SWDITMS, // Debug TMS
    output wire        SWDO,
    output wire        SWDOEN,
    input wire 	       CLK_1M,
    // Signals by dd
    input wire 	       RXEV,
    output wire        TXEV,
    
    // IO
    output wire [31:0] PA,
    output wire [31:0] PB,
    output wire [31:0] PC,
    output wire [31:0] PD,
    input wire [31:0]  PI,
    input wire [31:0]  PJ,
    input wire 	       TIN,
    output wire        TXD0,
    input wire 	       RXD0,
    output wire        TXD1,
    input wire 	       RXD1,
    output wire        TXD2,
    input wire 	       RXD2,
    output wire        BAUDTICK
   );

   // AHB connection
   wire [31:0] 	       HADDR;
   wire [31:0] 	       HWDATA;
   wire 	       HWRITE;
   wire [2:0] 	       HSIZE;
   wire [2:0] 	       HBURST;
   wire [3:0] 	       HPROT;
   wire [1:0] 	       HTRANS;
   wire 	       HREADY;
   		       
   wire [31:0] 	      oHRDATA;
   wire 	      oHREADY;
   wire 	      oHRESP;

   assign HREADY= oHREADY;
   assign AHB_HCLK= CLK;

   wire [31:0]  IRQ;
   
   dd_cpu cpu
     (
      .CLK(CLK),
      .PORESETn(RESETn),
      //.nTRST(1'b1/*nRST*/),
      .SWCLKTCK(SWCLKTCK),
      .SWDITMS(SWDITMS),
      .SWDO(SWDO),
      .SWDOEN(SWDOEN),
      .TDI(1'b0/*TDI*/),
      .TDO(/*TDO*/),
      .RXEV(RXEV),
      .TXEV(TXEV),
      .IRQ(IRQ),
      // AHB signals to drive outer items
      .HADDR(HADDR),
      .HWDATA(HWDATA),
      .HWRITE(HWRITE),
      .HSIZE(HSIZE),
      .HBURST(HBURST),
      .HPROT(HPROT),
      .HTRANS(HTRANS),
      // AHB answer from outsider-system
      .oHRDATA(oHRDATA),
      .oHREADY(oHREADY),
      .oHRESP(oHRESP)
      );

   osys
     #(
       .CODE_AW(CODE_AW),
       .DATA_AW(DATA_AW),
       .CODE_file(CODE_file)
       )
   outsider
     (
      .CLK(CLK),
      .RESETn(RESETn),
      .BOOT_MODE(BOOT_MODE),
      .CLK_1M(CLK_1M),
      // AHB signals driven by MCU
      .HADDR(HADDR),
      .HWDATA(HWDATA),
      .HWRITE(HWRITE),
      .HSIZE(HSIZE),
      .HBURST(HBURST),
      .HPROT(HPROT),
      .HTRANS(HTRANS),
      // Answer to MCU
      .oHRDATA(oHRDATA),
      .oHREADY(oHREADY),
      .oHRESP(oHRESP),
      // IRQ
      .IRQ(IRQ),
      // IO
      .PA(PA),
      .PB(PB),
      .PC(PC),
      .PD(PD),
      .PI(PI),
      .PJ(PJ),
      .TIN(TIN),
      .TXD0(TXD0),
      .RXD0(RXD0),
      .TXD1(TXD1),
      .RXD1(RXD1),
      .TXD2(TXD2),
      .RXD2(RXD2),
      .BAUDTICK(BAUDTICK)
      );
      
endmodule
