// Outside system, should react on addresses 0x5XXX_XXXX

module osys
  #(
    parameter CODE_AW=12,
    parameter CODE_file="",
    parameter DATA_AW=12
    )
   (
    input wire 	       CLK,
    input wire 	       RESETn,
    input wire 	       BOOT_MODE,
    input wire         CLK_1M,
    // AHB signals from MCU
    input wire [31:0]  HADDR,
    input wire [31:0]  HWDATA,
    input wire 	       HWRITE,
    input wire [2:0]   HSIZE,
    input wire [2:0]   HBURST,
    input wire [3:0]   HPROT,
    input wire [1:0]   HTRANS,
    // Answer to MCU
    output wire [31:0] oHRDATA,
    output wire        oHREADY,
    output wire        oHRESP,
    output wire        oHSEL,
    // IRQ
    output wire [31:0] IRQ,
    // IO
    output wire [31:0] PA,
    output wire [31:0] PB,
    output wire [31:0] PC,
    output wire [31:0] PD,
    input wire [31:0]  PI,
    input wire [31:0]  PJ,
    input wire 	       TIN,
    output wire        TXD0,
    input wire 	       RXD0,
    output wire        TXD1,
    input wire 	       RXD1,
    output wire        TXD2,
    input wire 	       RXD2,
    output wire        BAUDTICK
    );
   
   wire [3:0] 	       mux_select;
   wire 	       HREADY;
   
   // Connections of code_memory
   wire 	       HSEL_code;
   wire [31:0] 	       HRDATA_code;
   wire 	       HREADYOUT_code;
   wire 	       HRESP_code;
   
   // Connections of periph0: output port
   wire 	       HSEL_oporto;
   wire [31:0] 	       HRDATA_oporto;
   wire 	       HREADYOUT_oporto;
   wire 	       HRESP_oporto;
   
   // Connections of periph1: watch
   wire 	       HSEL_watch;
   wire [31:0] 	       HRDATA_watch;
   wire 	       HREADYOUT_watch;
   wire 	       HRESP_watch;
   
   // Connections of periph2: input port
   wire 	       HSEL_iport;
   wire [31:0] 	       HRDATA_iport;
   wire 	       HREADYOUT_iport;
   wire 	       HRESP_iport;
   
   // Connections of apb subsytem (timer, uart)
   wire 	       HSEL_apb;
   wire [31:0] 	       HRDATA_apb;
   wire 	       HREADYOUT_apb;
   wire 	       HRESP_apb;
   
   // Connections of data memory
   wire 	       HSEL_data;
   wire [31:0] 	       HRDATA_data;
   wire 	       HREADYOUT_data;
   wire 	       HRESP_data;

   // Connections of system rom table
   wire 	       HSEL_srom;
   wire [31:0] 	       HRDATA_srom;
   wire 	       HREADYOUT_srom;
   wire 	       HRESP_srom;
   
   // Connections of test module
   /*
   wire 	       HSEL_test;
   wire [31:0] 	       HRDATA_test;
   wire 	       HREADYOUT_test;
   wire 	       HRESP_test;
   */
   
   assign oHREADY= HREADY;
      
   ahb_decmux decmux
     (
      .HCLK(CLK),
      .HADDR(HADDR),
      .HRESETn(RESETn),
      .BOOT_MODE(BOOT_MODE),
      
      .HRDATA_NOMAP(32'hDEADBEEF),
      .HREADYOUT_NOMAP(1'b1),
      .HRESP_NOMAP(1'b1),

      .HRDATA_CODE(HRDATA_code),
      .HREADYOUT_CODE(HREADYOUT_code),
      .HRESP_CODE(HRESP_code),
      .HSEL_CODE(HSEL_code),

      .HRDATA_DATA(HRDATA_data),
      .HREADYOUT_DATA(HREADYOUT_data),
      .HRESP_DATA(HRESP_data),
      .HSEL_DATA(HSEL_data),

      .HRDATA_P0(HRDATA_oporto),
      .HREADYOUT_P0(HREADYOUT_oporto),
      .HRESP_P0(HRESP_oporto),
      .HSEL_P0(HSEL_oporto),

      .HRDATA_P1(HRDATA_watch),
      .HREADYOUT_P1(HREADYOUT_watch),
      .HRESP_P1(HRESP_watch),
      .HSEL_P1(HSEL_watch),

      .HRDATA_P2(HRDATA_iport),
      .HREADYOUT_P2(HREADYOUT_iport),
      .HRESP_P2(HRESP_iport),
      .HSEL_P2(HSEL_iport),

      .HRDATA_APB(HRDATA_apb),
      .HREADYOUT_APB(HREADYOUT_apb),
      .HRESP_APB(HRESP_apb),
      .HSEL_APB(HSEL_apb),

      .HRDATA_SROM(HRDATA_srom),
      .HREADYOUT_SROM(HREADYOUT_srom),
      .HRESP_SROM(HRESP_srom),
      .HSEL_SROM(HSEL_srom),
        
      .HRDATA_TEST(/*HRDATA_test*/32'hdeadbeef),
      .HREADYOUT_TEST(/*HREADYOUT_test*/1'b1),
      .HRESP_TEST(/*HRESP_test*/1'b0),
      .HSEL_TEST(/*HSEL_test*/),
        
      .HRDATA(oHRDATA),
      .HREADYOUT(HREADY),
      .HRESP(oHRESP),
      .HSEL_ANY(oHSEL)
      );
      
   //ahb_rom
   //cmsdk_ahb_ram_beh
   ahb_mem4
     #(
       .AW(CODE_AW),
       .MEMFILE(CODE_file)
       )
   mcode
     (
      .HCLK       (CLK),
      .HRESETn    (RESETn),
      .HSEL       (HSEL_code),
      .HADDR      (HADDR),
      .HTRANS     (HTRANS),
      .HREADY     (HREADY),
      .HWRITE	  (HWRITE),
      .HSIZE      (HSIZE),
      .HWDATA     (HWDATA),
      .HREADYOUT  (HREADYOUT_code), // Outputs
      .HRDATA     (HRDATA_code),
      .HRESP      (HRESP_code)
      );

   ahb_fpga_sram
     #(
       //.MEM_TYPE(2),
       .AW(DATA_AW)
     )
   mdata
     (
      .HCLK       (CLK),
      .HRESETn    (RESETn),
      .HSEL       (HSEL_data),
      .HADDR      (HADDR[DATA_AW-1:0]),
      .HTRANS     (HTRANS),
      .HWRITE     (HWRITE),
      .HSIZE      (HSIZE),
      .HWDATA     (HWDATA),
      .HREADY     (HREADY),
      .HREADYOUT  (HREADYOUT_data),
      .HRDATA     (HRDATA_data),
      .HRESP      (HRESP_data)
      );

   // Oport on outside system
   /*oport4*//*cmsdk_ahb_eg_slave*/
   ahb_out4 #(.ADDRWIDTH(12)) oporto
     (
      .HCLK       (CLK),
      .HRESETn    (RESETn),
      .ECOREVNUM  (4'h5),
      .HSEL       (HSEL_oporto),
      .HADDR      (HADDR[11:0]),
      .HTRANS     (HTRANS),
      .HWRITE     (HWRITE),
      .HSIZE      (HSIZE),
      .HWDATA     (HWDATA),
      .HREADY     (HREADY),
      .HREADYOUT  (HREADYOUT_oporto),
      .HRDATA     (HRDATA_oporto),
      .HRESP      (HRESP_oporto),
      .r0	  (PA),
      .r1	  (PB),
      .r2	  (PC),
      .r3	  (PD)
      );

   // Watch on outside system
   ahb_watch_v2 #(.ADDRWIDTH(12)) watch
     (
      .HCLK       (CLK),
      .HRESETn    (RESETn),
      .ECOREVNUM  (4'h6),
      .HSEL       (HSEL_watch),
      .HADDR      (HADDR[11:0]),
      .HTRANS     (HTRANS),
      .HWRITE     (HWRITE),
      .HSIZE      (HSIZE),
      .HWDATA     (HWDATA),
      .HREADY     (HREADY),
      .HREADYOUT  (HREADYOUT_watch),
      .HRDATA     (HRDATA_watch),
      .HRESP      (HRESP_watch)
      );

   // Iport on outside system
   ahb_in2 #(.ADDRWIDTH(12)) iport
     (
      .HCLK       (CLK),
      .HRESETn    (RESETn),
      .ECOREVNUM  (4'h6),
      .HSEL       (HSEL_iport),
      .HADDR      (HADDR[11:0]),
      .HTRANS     (HTRANS),
      .HWRITE     (HWRITE),
      .HSIZE      (HSIZE),
      .HWDATA     (HWDATA),
      .HREADY     (HREADY),
      .HREADYOUT  (HREADYOUT_iport),
      .HRDATA     (HRDATA_iport),
      .HRESP      (HRESP_iport),
      .pi	  (PI),
      .pj	  (PJ)
      );

  cm0ik_system_rom_table
   #(.BASE                              (32'hF0000000))
   u_system_rom_table
    (//Outputs
     .HRDATA                            (HRDATA_srom),
     .HREADYOUT                         (HREADYOUT_srom),
     .HRESP                             (HRESP_srom),
     //Inputs
     .HCLK                              (CLK),
     .HSEL                              (HSEL_srom),
     .HADDR                             (HADDR[12:0]),
     .HTRANS                            (HTRANS),
     .HREADY                            (HREADY),
     .CUSTJEPID                         (7'b0),          
     .CUSTJEPCONTINUATION               (4'b0),
     .CUSTPARTNUMBER                    (12'b0),    
     .CUSTREVISION                      (4'b0),     
     .CUSTREVAND                        (4'b0),      
     .SIVREVAND                         (4'b0)
     );     

   wire 	       apbactive;
   wire 	       pclkg;
   wire [31:0] apb_irqs;
   
   cmsdk_clock_gate apb_cg
     (
      .CLK		(CLK),
      .DISABLEG		(1'b0),
      .CLKENABLE	(apbactive),
      .GATEDCLK		(pclkg)
      );
   
   cmsdk_apb_subsystem apb_sys
     (
      .HCLK			(CLK),
      .HRESETn			(RESETn),

      .PCLK			(CLK),
      .PCLKG			(pclkg),
      .APBACTIVE		(apbactive),
      .PCLKEN			(1'b1),
      .PRESETn			(RESETn),
      
      .HSEL			(HSEL_apb),
      .HADDR			(HADDR[15:0]),
      .HTRANS			(HTRANS),
      .HWRITE			(HWRITE),
      .HSIZE			(HSIZE),
      .HWDATA			(HWDATA),
      .HREADY			(HREADY),
      .HPROT                    (HPROT),
      .HREADYOUT		(HREADYOUT_apb),
      .HRDATA			(HRDATA_apb),
      .HRESP			(HRESP_apb),

      .timer0_extin		(TIN),
      
      .uart0_tick		(BAUDTICK),
      .uart0_rxd		(RXD0),
      .uart0_txd		(TXD0),
      .uart1_rxd		(RXD1),
      .uart1_txd		(TXD1),
      .uart2_rxd		(RXD2),
      .uart2_txd		(TXD2),
      
      .apbsubsys_interrupt	(apb_irqs)
      );
   
   assign IRQ= {16'd0, apb_irqs[15:0]};
   
endmodule // osys
