module watch
  #(
    parameter AW=12
    )
   (
   // clock and reset
   input wire 	       hclk,
   input wire 	       hresetn,
   // register interface
   input wire [AW-1:0] addr,
   input wire 	       read_en,
   input wire 	       write_en,
   input wire [3:0]    byte_strobe,
   input wire [31:0]   wdata,
   input wire [3:0]    ecorevnum,
   output wire [31:0]  rdata,
    // IO signals
   input wire 	       clk_1m
   );

   localparam RN_CNT_L= 3'h0;
   localparam RN_CNT_H= 3'h1;
   localparam RN_BUF= 3'h2;
   localparam RN_STAT= 3'h3;
   
   // Counter, 2 register wide
   reg [63:0] 	       cnt;
   // Write buffer
   reg [31:0] 	       wbufh;
   reg [31:0] 	       wbufl;
   // Read buffer
   reg [31:0] 	       rbufh;
   reg [31:0] 	       rbufl;

   reg 		       wrdy_can_cleared;
   wire 	       wrdy_set, wrdy_res, wbuf_ready;

   wire [2:0]	       rn;
   assign rn= addr[4:2];
   
   assign wrdy_set= write_en && (rn == RN_CNT_H) && hclk;
   assign wrdy_res= wrdy_can_cleared;//wbuf_ready && clk_1m;
   
   rs wrdy
     (
      .res(!hresetn),
      .s(wrdy_set),
      .r(wrdy_res),
      .q(wbuf_ready)
      );

   always @(posedge clk_1m)
     if (!hresetn)
       wrdy_can_cleared<= 0;
     else
       begin
	  if (wbuf_ready && !wrdy_set)
	    wrdy_can_cleared<= 1;
	  else
	    wrdy_can_cleared<= 0;
       end
   
   always @(posedge clk_1m)
     begin
	if (!hresetn)
	  cnt<= 0;
	else
	  begin
	     if (wbuf_ready)
	       cnt<= {wbufh,wbufl};
	     else
	       cnt<= cnt+1;
	  end
     end

   always @(posedge hclk)
     begin
	if (write_en && (rn == RN_BUF))
	  begin
	     rbufh<= cnt[63:32];
	     rbufl<= cnt[31:0];
	  end
     end

   always @(posedge hclk)
     begin
	if (write_en && (rn == RN_CNT_L))
	  begin
	     if (byte_strobe[0])
	       wbufl[7:0]<= wdata[7:0];
	     if (byte_strobe[1])
	       wbufl[15:8]<= wdata[15:8];
	     if (byte_strobe[2])
	       wbufl[23:16]<= wdata[23:16];
	     if (byte_strobe[3])
	       wbufl[31:24]<= wdata[31:24];
	  end
     end
   
   always @(posedge hclk)
     begin
	if (write_en && (rn == RN_CNT_H))
	  begin
	     if (byte_strobe[0])
	       wbufh[7:0]<= wdata[7:0];
	     if (byte_strobe[1])
	       wbufh[15:8]<= wdata[15:8];
	     if (byte_strobe[2])
	       wbufh[23:16]<= wdata[23:16];
	     if (byte_strobe[3])
	       wbufh[31:24]<= wdata[31:24];
	  end
     end

   /*
   always @(read_en, addr, rbufh, rbufl)
     begin
	if (read_en)
	  begin
	     case (addr[4:2])
	       ADDR_CNT_L:
		 rdata= rbufl;
	       ADDR_CNT_H:
		 rdata= rbufh;
	     endcase
	  end
	else
	  rdata= {32{1'bx}};
     end
   */

   wire [31:0] status;
   wire        busy;

   assign busy= wrdy_set | wrdy_res | wbuf_ready;
   assign status= {28'h0,
		   wrdy_set,
		   wrdy_res,
		   wbuf_ready,
		   busy
		   };
   
   assign rdata
     =
      !read_en?{32{1'b0}}:
      (
       (rn==RN_CNT_L)?rbufl:
       (rn==RN_CNT_H)?rbufh:
       (rn==RN_STAT)?status:
       32'h0
       );
   
endmodule // watch
