module watch_v2
  #(
    parameter AW=12
    )
   (
    // clock and reset
    input wire 		hclk,
    input wire 		hresetn,
    // register interface
    input wire [AW-1:0] addr,
    input wire 		read_en,
    input wire 		write_en,
    input wire [3:0] 	byte_strobe,
    input wire [31:0] 	wdata,
    input wire [3:0] 	ecorevnum,
    output wire [31:0] 	rdata
    );
   
   localparam RN_CNT_L= 3'd0;
   localparam RN_CNT_H= 3'd1;
   localparam RN_BUF  = 3'd2;
   localparam RN_STAT = 3'd3;
   localparam RN_CTRL = 3'd4;
   localparam RN_PRE  = 3'd5;
   
   // Counter, 2 register wide
   reg [63:0] 	       cnt= 0;

   // Control register
   reg [31:0] 	       ctrl= 0;
   wire 	       ctrl_en= ctrl[0];

   // Pre-counter value
   reg [31:0] 	       pre= 0;
       
   // Read buffer
   reg [31:0] 	       rbufh= 0;
   reg [31:0] 	       rbufl= 0;

   wire [2:0]	       rn;
   assign rn= addr[4:2];
   wire 	       write4= write_en && (&byte_strobe);

   reg [15:0] 	       pre_cnt= 0;
   wire 	       pre_end= (pre_cnt>=(pre[15:0]-1));
   wire 	       dived= (pre==0)?hclk:pre_end;

   always @(posedge hclk)
     if (!hresetn)
       pre_cnt<= 0;
     else
       if (pre_end)
	 pre_cnt<= 0;
       else
	 pre_cnt<= pre_cnt+1;
   
   always @(posedge hclk)
     begin
	if (!hresetn)
	  cnt<= 0;
	else
	  begin
	     if (write4)
	       begin
		  if (rn==RN_CNT_L)
		    cnt<= { cnt[63:32], wdata };
		  else if (rn==RN_CNT_H)
		    cnt<= { wdata, cnt[31:0] };
	       end
	     if (ctrl_en && dived)
	       cnt<= cnt+1;
	  end
     end

   always @(posedge hclk)
     begin
	if (!hresetn)
	  begin
	     rbufh<= 32'h0;
	     rbufl<= 32'h0;
	  end
	else
	  if (write4 && (rn == RN_BUF))
	    begin
	       rbufh<= cnt[63:32];
	       rbufl<= cnt[31:0];
	    end
     end

   always @(posedge hclk)
     begin
	if (!hresetn)
	  ctrl<= 32'h0;
	else
	  if (write4 && (rn == RN_CTRL))
	    ctrl<= wdata;
     end

   always @(posedge hclk)
     begin
	if (!hresetn)
	  pre<= 32'h0;
	else
	  if (write4 && (rn == RN_PRE))
	    pre<= wdata;
     end

   wire [31:0] status;
   wire        busy;

   assign status= {32'h0
		   };
   
   assign rdata
     =
      !read_en?{32{1'b0}}:
      (
       (rn==RN_CNT_L)?rbufl:
       (rn==RN_CNT_H)?rbufh:
       (rn==RN_CTRL)?ctrl:
       (rn==RN_STAT)?status:
       (rn==RN_PRE)?pre:
       32'h0
       );
   
endmodule // watch
