TC		= arm-none-eabi
CC		= $(TC)-gcc
CXX		= $(TC)-g++
LD		= $(TC)-ld
AR		= $(TC)-ar
STRIP		= $(TC)-strip
RANLIB		= $(TC)-ranlib
SIZE		= $(TC)-size
OBJCOPY		= $(TC)-objcopy
GDB		= $(TC)-gdb
OD		= $(TC)-objdump
OC		= $(OBJCOPY)

CP		= cp

ifeq ($(OS),Windows_NT)
RM		= del /f /q
RMR		= del /f /s /q
ISS		= $(TOOLS)/sim.bat
else
RM		= rm -f
RMR		= rm -f -r
ISS		= $(TOOLS)/sim.sh
endif

LINK_CMD	?= arm-gcc.ld

ARMFLAGS	= -mthumb -mcpu=cortex-m0 -mfloat-abi=soft

INCLUDES	= -I$(PRJ) -I. \
		-I$(PRJ)/utils \
		-I$(PRJ)/cmsis \
		-I$(PRJ)/cmsdk \
		-I$(PRJ)/driver

CFLAGS		= -O2 $(ARMFLAGS) -save-temps -Wa,-adhln -fverbose-asm -ffunction-sections

CFLAGS		+= -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.o)"

CXXFLAGS	= -O0 -g $(ARMFLAGS) -Wa,-adhln -fverbose-asm -ffunction-sections

LDFLAGS		= $(ARMFLAGS) -Wl,-Map=$@.map -Wl,-T$(LINK_CMD) -Wl,--gc-sections -Wl,--cref 
#LDFLAGS		= $(ARMFLAGS) -Wl,-Map=$@.map -Wl,-T$(LINK_CMD) -Wl,--gc-sections -nostartfiles -Wl,--cref -lgcc -lc

CPPFLAGS	= $(DEFINES) $(INCLUDES)


.SUFFIXES: .s .hex .elf .bin .asc .m8h .0as .1as .2as .3as

.s.o:
	@echo AS $<
	@$(AS) -o $@ $<

SILENT	?= yes

.c.o:
ifeq ($(SILENT),yes)
	@echo CC $<
	@$(CC) $(CFLAGS) $(CPPFLAGS) -c -o $@ $< >$<.lst
else
	$(CC) $(CFLAGS) $(CPPFLAGS) -c -o $@ $< >$<.lst
endif

.elf.hex:
	$(OBJCOPY) -O ihex $< $@

.elf.bin:
	$(OBJCOPY) -O binary $< $@

.bin.0as:
	@php tools/bin2as.php -aw $(CODE_AW) -s 0 $< >$@

.bin.1as:
	@php tools/bin2as.php -aw $(CODE_AW) -s 1 $< >$@

.bin.2as:
	@php tools/bin2as.php -aw $(CODE_AW) -s 2 $< >$@

.bin.3as:
	@php tools/bin2as.php -aw $(CODE_AW) -s 3 $< >$@

#include $(wildcard *.d)
#include $(wildcard $(PRJ)/*.d)
#include $(wildcard $(PRJ)/utils/*.d)
