#include <stdio.h>

#include "drv_uart.h"


static ALIGN volatile unsigned int rbuf0_avail;
static ALIGN volatile unsigned int uart0_sends;
static ALIGN volatile unsigned int rbuf1_avail;
static ALIGN volatile unsigned int uart1_sends;
static ALIGN volatile unsigned int rbuf2_avail;
static ALIGN volatile unsigned int uart2_sends;

static struct dev_uart_t dev_uart[nuof_uarts]=
  {
    // 0
    {
      .inited= 0,
      .UART=UART0,
      .RX_IRQn= UART0_RX_IRQn,
      .TX_IRQn= UART0_TX_IRQn,
      .rff= 0,
      .rlu= 0,
      .avail= &rbuf0_avail,
      .sends= &uart0_sends
    },
    // 1
    {
      .inited=0,
      .UART=
#ifdef UART1
      UART1
#else
      NULL
#endif
    ,
      .RX_IRQn= UART1_RX_IRQn,
      .TX_IRQn= UART1_TX_IRQn,
      .rff= 0,
      .rlu= 0,
      .avail= &rbuf1_avail,
      .sends= &uart1_sends
    },
    // 2
   {
     .inited=0,
     .UART=
#ifdef UART2
     UART2
#else
     NULL
#endif
     ,
     .RX_IRQn= UART2_RX_IRQn,
     .TX_IRQn= UART2_TX_IRQn,
     .rff= 0,
     .rlu= 0,
     .avail= &rbuf2_avail,
     .sends= &uart2_sends
   }
  };

//char rbuf0[1];

static struct dev_uart_t *
get_dev(int unr)
{
  if (unr >= nuof_uarts)
    return NULL;
  if (dev_uart[unr].UART == NULL)
    return 0;
  return &dev_uart[unr];
}

void
rx_handler(int unr)
{
  struct dev_uart_t *dev;
  unsigned char c;
  if ((dev= get_dev(unr))==NULL)
    return;
  dev->UART->INTSTATUS= 2; // clear RX irq
  c= dev->UART->DATA;
  //rbuf0[0]= c;
  *(dev->avail)= 1;
  {
    int n= (dev->rff+1)%rbuf_size;
    if (n != dev->rlu)
      {
	dev->rbuf[dev->rff]= c;
	dev->rff= n;
      }
  }
}

void
UART0_RX_handler(void)
{
  rx_handler(0);
}

void
UART1_RX_handler(void)
{
  rx_handler(1);
}

void
UART2_RX_handler(void)
{
  rx_handler(2);
}

void
tx_handler(int unr)
{
  struct dev_uart_t *dev;
  if ((dev= get_dev(unr))==NULL)
    return;
  dev->UART->INTSTATUS= 1;
  if (dev->tlu == dev->tff)
    *(dev->sends)= 0;
  else
    {
      char c;
      c= dev->tbuf[dev->tlu];
      dev->tlu= (dev->tlu+1) % tbuf_size;
      dev->UART->DATA= c;
    }
}

void
UART0_TX_handler(void)
{
  tx_handler(0);
}

void
UART1_TX_handler(void)
{
  tx_handler(1);
}

void
UART2_TX_handler(void)
{
  tx_handler(2);
}

int
uart_init(int unr)
{
  unsigned int u;
  struct dev_uart_t *dev;

  if ((dev= get_dev(unr))==NULL)
    return -1;
  cmsdk_uart_init(dev->UART, 27);
  cmsdk_uart_rxen(dev->UART, 1);
  cmsdk_uart_txen(dev->UART, 1);

  *(dev->avail)= 0;
  *(dev->sends)= 0;
  u= dev->UART->CTRL;
  u|= 8; // RX IRQ enable
  u|= 4; // TX IRQ enable
  dev->UART->CTRL= u;

  NVIC_SetPriority(dev->RX_IRQn, 1);
  NVIC_EnableIRQ(dev->RX_IRQn);    
  NVIC_SetPriority(dev->TX_IRQn, 1);
  NVIC_EnableIRQ(dev->TX_IRQn);
  __enable_irq();
  dev->inited= 1;
  return 0;
}

int
uart_close(int unr)
{
  struct dev_uart_t *dev;
  if ((dev= get_dev(unr))==NULL)
    return -1;
  cmsdk_uart_rxen(dev->UART, 0);
  cmsdk_uart_txen(dev->UART, 0);
  dev->inited= 0;
  return 0;
}

int
uart_write(int unr, char *buf, int len)
{
  int i= 0;
  struct dev_uart_t *dev;
  if ((dev= get_dev(unr))==NULL)
    return -1;
  if (!dev->inited)
    uart_init(unr);
  if (buf && len)
    {
      int lu, ff, n;
      NVIC_DisableIRQ(dev->TX_IRQn);
      lu= dev->tlu;
      ff= dev->tff;
      n= (ff+1) % tbuf_size;
      i= 0;
      while ((n!=lu) && (i<len))
	{
	  char c;
	  c= buf[i];
	  dev->tbuf[ff]= c;
	  ff= n;
	  n= (n+1) % tbuf_size;
	  i++;
	}
      dev->tff= ff;
      if (!(*(dev->sends)))
	{
	  char c;
	  *(dev->sends)= 1;
	  c= dev->tbuf[dev->tlu];
	  dev->tlu= (dev->tlu+1) % tbuf_size;
	  dev->UART->INTSTATUS= 1;
	  dev->UART->DATA= c;
	}
      NVIC_EnableIRQ(dev->TX_IRQn);    
    }
  return i;
}


unsigned int
uart_avail(int unr)
{
  struct dev_uart_t *dev;
  if ((dev= get_dev(unr))==NULL)
    return 0;
  if (!dev->inited)
    uart_init(unr);
  return *(dev->avail);
}

unsigned char
uart_receive(int unr)
{
  char c;
  //c= UART0->DATA;
  struct dev_uart_t *dev;
  if ((dev= get_dev(unr))==NULL)
    return -1;
  if (!dev->inited)
    uart_init(unr);
  {
    NVIC_DisableIRQ(dev->RX_IRQn);
    {
      //c= rbuf0[0];
      if (dev->rff != dev->rlu)
	{
	  c= dev->rbuf[dev->rlu];
	  dev->rlu= (dev->rlu+1) % rbuf_size;
	}
      *(dev->avail)= dev->rlu != dev->rff;
    }
    NVIC_EnableIRQ(dev->RX_IRQn);    
  }
  return c;
}
