#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "fs.h"

#include "drv_uart.h"


unsigned char fs_files[3+fs_nuof_files];

unsigned char
fs_name2drv(const char *name)
{
  if (!name || !*name)
    return fs_none;
  if (strstr(name, "/dev/uart") == name)
    {
      int d;
      const char *p= name + 9;
      d= strtol(p, 0, 10);
      d&= fs_dev_mask;
      return fs_uart + d;
    }
  if (strstr(name, "/dev/zero") == name)
    {
      return fs_vsys + fs_zero;
    }
}


int
drv_open(unsigned char drv)
{
  int dev= drv & fs_dev_mask;
  switch (drv & fs_drv_mask)
    {
    case fs_uart:
      uart_init(dev);
      break;
    case fs_vsys:
      switch (dev)
	{
	case fs_zero: break;
	}
      break;
    }
  
  return 0;
}

int
drv_close(unsigned char drv)
{
  int dev= drv & fs_dev_mask;
  switch (drv & fs_drv_mask)
    {
    case fs_uart:
      uart_close(dev);
      break;
    case fs_vsys:
      switch (dev)
	{
	case fs_zero: break;
	}
      break;
    }
  return 0;
}


int
fs_stdio_init(char *fin, char *fout, char *ferror)
{
  int fd;
  unsigned char drv;

  if (fin && *fin)
    {
      if (fs_files[0] != fs_none)
	{
	  if (stdin != NULL)
	    {
	      fclose(stdin);
	      stdin= NULL;
	    }
	  else
	    close(0);
	}
      drv= fs_name2drv(fin);
      fs_files[0]= drv;
      stdin= fdopen(0, "r");
      /*tcgetattr(0, &tp);
      tp.c_lflag|= ECHO;
      cfsetspeed(&tp, 115200);
      tcsetattr(0, TCSANOW, &tp);*/
    }

  if (fout && *fout)
    {
      if (fs_files[1] != fs_none)
	{
	  if (stdout != NULL)
	    {
	      fclose(stdout);
	      stdout= NULL;
	    }
	  else
	    close(1);
	}
      drv= fs_name2drv(fout);
      fs_files[1]= drv;
      stdout= fdopen(1, "w");
      /*tcgetattr(1, &tp);
      cfsetspeed(&tp, 115200);
      tcsetattr(1, TCSANOW, &tp);*/
    }

  if (ferror && *ferror)
    {
      if (fs_files[2] != fs_none)
	{
	  if (stderr != NULL)
	    {
	      fclose(stderr);
	      stderr= NULL;
	    }
	  else
	    close(2);
	}
      drv= fs_name2drv(ferror);
      fs_files[2]= drv;
      stderr= fdopen(2, "w");
      /*tcgetattr(1, &tp);
      cfsetspeed(&tp, 115200);
      tcsetattr(1, TCSANOW, &tp);*/
    }

  return 0;
}

int
fs_open(const char *fn, int fl, int m)
{
  unsigned char drv= fs_name2drv(fn);
  int dev= drv & fs_dev_mask;
  int fno, i;
  for (fno= 3; fno < 3+fs_nuof_files; fno++)
    if (fs_files[fno] == fs_none)
      break;
  if (fno >= 3+fs_nuof_files)
    return -1;
  
  i= drv_open(drv);
  if (i == 0)
    fs_files[fno]= drv;
  else
    return -2;
  
  return fno;
}

int
fs_write(int fid, char *buf, int len)
{
  unsigned char drv;
  int dev;
  if (fid >= 3+fs_nuof_files)
    return -1;
  drv= fs_files[fid];
  dev= drv & fs_dev_mask;
  switch (drv & fs_drv_mask)
    {
    case fs_uart:
      return uart_write(dev, buf, len);
    case fs_vsys:
      switch (dev)
	{
	case fs_zero: return len;
	}
      break;
    }
  return 0;
}

int
fs_read(int fid, char *buf, int len)
{
  return -1;
}

int
fs_close(int fid)
{
  unsigned char drv;
  if (fid >= 3+fs_nuof_files)
    return -1;
  drv= fs_files[fid];
  drv_close(drv);
  fs_files[fid]= fs_none;
  return 0;
}
