OUTPUT_FORMAT ("elf32-littlearm", "elf32-bigarm", "elf32-littlearm")
/*
MEMORY { rom	:  ORIGIN = 0x00000000, LENGTH = 128k }
MEMORY { ram	:  ORIGIN = 0x20000000, LENGTH = 64k }
*/
INCLUDE pro_code.ld
INCLUDE pro_data.ld

SECTIONS
{
	. = ORIGIN(rom);

	.vectors :
    	{
		KEEP(*(.vectors))
    	} >rom

	.text :
    	{
		*(.text)
		*(.text.*)
		*(.glue_7t)
		*(.glue_7)
		KEEP(*(.init))
		KEEP(*(.fini))
		/*KEEP(*(.jcr*))*/
	} >rom

	.rodata :
	{
		. = ALIGN(4);
		*(.rodata)
		*(.rodata*)
	} >rom
	
	.ARM.extab :
	{
		*(.ARM.extab* .gnu.linkonce.armextab.*)
	} > rom

	.ARM.exidx :
	{
		__exidx_start = .;
		*(.ARM.exidx* .gnu.linkonce.armexidx.*)
		__exidx_end = .;
	} > rom

	.preinit_array :
	{
		PROVIDE_HIDDEN (__preinit_array_start = .);
		KEEP(*(.preinit_array))
		PROVIDE_HIDDEN (__preinit_array_end = .);
	} >rom

	.init_array :
	{
		PROVIDE_HIDDEN (__init_array_start = .);
		KEEP(*(SORT(.init_array.*)))
		KEEP(*(.init_array))
		PROVIDE_HIDDEN (__init_array_end = .);
	} >rom

	.fini_array :
	{
		PROVIDE_HIDDEN (__fini_array_start = .);
		KEEP(*(SORT(.fini_array.*)))
		KEEP(*(.fini_array))
		PROVIDE_HIDDEN (__fini_array_end = .);
	} >rom
	
	_end_text = .;
    	_sidata = LOADADDR(.data);
    
	.data : /*AT (_end_text)*/
    	{
		_start_data = .;
        	*(.data)
		*(.data.*)
		. = ALIGN(4);
		_end_data = .;
	} >ram AT> rom

	.bss (NOLOAD) :
    	{
		. = ALIGN(4);
        	_start_bss = .;
		__bss_start = .;
		__bss_start__ = .;
        	*(.bss .bss.*)
		*(COMMON)
		. = ALIGN(4);
        	_end_bss = .;
		__bss_end__ = .;
    	} >ram

	.omem (NOLOAD) :
    	{
		*(.omem)
		*(.omem.*)
    	} >ram

	._user_heap :
    	{
		. = ALIGN(8);
		PROVIDE ( end = . );
		PROVIDE ( _end = . );
    	} >ram
    
	.co_stack (NOLOAD) :
    	{
		_start_stack = .;
		*(.co_stack .co_stack.*)
        	_end_stack = ORIGIN(ram) + LENGTH(ram);
		_eram = _end_stack;
	} >ram

	/* Remove information from the standard libraries */
  	/DISCARD/ :
  	{
		libc.a ( * )
    		libm.a ( * )
    		libgcc.a ( * )
  	}

	.ARM.attributes 0 : { *(.ARM.attributes) }
  
	. = ALIGN(4);
    	_end = .;
}
