#include <stdint.h>

#include "config.h"

/* Addresses pulled in from the linker script */
extern uint32_t _end_stack;
extern uint32_t _end_text;
extern uint32_t _start_data;
extern uint32_t _end_data;
extern uint32_t _start_bss;
extern uint32_t _end_bss;
extern uint32_t _sidata;

/* Application main() called in reset handler */
extern int main(void);

#define WEAK_ALIAS(x) __attribute__ ((weak, alias(#x)))

/* Cortex M0 core interrupt handlers */
void Reset_Handler(void);
void NMI_Handler(void) WEAK_ALIAS(Dummy_Handler);
void HardFault_Handler(void) WEAK_ALIAS(Dummy_Handler);
void MemManage_Handler(void) WEAK_ALIAS(Dummy_Handler);
void BusFault_Handler(void) WEAK_ALIAS(Dummy_Handler);
void UsageFault_Handler(void) WEAK_ALIAS(Dummy_Handler);
void SVC_Handler(void) WEAK_ALIAS(Dummy_Handler);
void DebugMon_Handler(void) WEAK_ALIAS(Dummy_Handler);
void PendSV_Handler(void) WEAK_ALIAS(Dummy_Handler);
void SysTick_Handler(void) WEAK_ALIAS(Dummy_Handler);

/* specific interrupt handlers */
//void WAKEUP_Handler(void) WEAK_ALIAS(Dummy_Handler);
void UART0_RX_handler(void) WEAK_ALIAS(Dummy_Handler);
void UART0_TX_handler(void) WEAK_ALIAS(Dummy_Handler);
void UART1_RX_handler(void) WEAK_ALIAS(Dummy_Handler);
void UART1_TX_handler(void) WEAK_ALIAS(Dummy_Handler);
void UART2_RX_handler(void) WEAK_ALIAS(Dummy_Handler);
void UART2_TX_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq6_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq7_handler(void) WEAK_ALIAS(Dummy_Handler);
void TIMER0_handler(void) WEAK_ALIAS(Dummy_Handler);
void TIMER1_handler(void) WEAK_ALIAS(Dummy_Handler);
void TIMER2_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq11_handler(void) WEAK_ALIAS(Dummy_Handler);
void UART0_OVF_handler(void) WEAK_ALIAS(Dummy_Handler);
void UART1_OVF_handler(void) WEAK_ALIAS(Dummy_Handler);
void UART2_OVF_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq15_handler(void) WEAK_ALIAS(Dummy_Handler);

void irq16_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq17_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq18_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq19_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq20_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq21_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq22_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq23_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq24_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq25_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq26_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq27_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq28_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq29_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq30_handler(void) WEAK_ALIAS(Dummy_Handler);
void irq31_handler(void) WEAK_ALIAS(Dummy_Handler);

void Dummy_Handler(void);

/* Stack top and vector handler table */
void *vector_table[] __attribute__ ((section(".vectors"))) = {
    &_end_stack,
    Reset_Handler,
    NMI_Handler,
    HardFault_Handler,
    MemManage_Handler,
    BusFault_Handler,
    UsageFault_Handler,
    0,
    0,
    0,
    0,
    SVC_Handler,
    DebugMon_Handler,
    0,
    PendSV_Handler,
    SysTick_Handler,

    /* specific interrupt vectors */
    UART0_RX_handler,
    UART0_TX_handler,
    UART1_RX_handler,
    UART1_TX_handler,
    UART2_RX_handler,
    UART2_TX_handler,
    irq6_handler,
    irq7_handler,
    TIMER0_handler,
    TIMER1_handler,
    TIMER2_handler,
    irq11_handler,
    UART0_OVF_handler,
    UART1_OVF_handler,
    UART2_OVF_handler,
    irq15_handler,
    
    irq16_handler,
    irq17_handler,
    irq18_handler,
    irq19_handler,
    irq20_handler,
    irq21_handler,
    irq22_handler,
    irq23_handler,
    irq24_handler,
    irq25_handler,
    irq26_handler,
    irq27_handler,
    irq28_handler,
    irq29_handler,
    irq30_handler,
    irq31_handler
};

#include <stdio.h>
#include "dev.h"

//struct _reent glob_reent;

void Reset_Handler(void) {
    uint8_t *src, *dst;
    unsigned int i;
#ifdef DEBUG_STARTUP
    PC= 0x11;
#endif
    //_impure_ptr= &glob_reent;
    /* Copy with byte pointers to obviate unaligned access problems */
    extern char *heap_end;
    heap_end= (char*)0;
    
#ifndef SKIP_COPY_DATA    
    /* Copy data section from Flash to RAM */
    //src = (uint8_t *)&_end_text;
    src = (uint8_t *)&_sidata;
    dst = (uint8_t *)&_start_data;
    while (dst < (uint8_t *)&_end_data)
      {
	*dst++ = *src++;
#ifdef DEBUG_STARTUP
	PA= (uint32_t)dst;
	PB= (uint32_t)src;
#endif
      }
#endif
    
    
#ifdef DEBUG_STARTUP
    PC= 0x22;
    PB= 0;
#endif


#ifndef SKIP_CLEAR_BSS    
    /* Clear the bss section */
    dst = (uint8_t *)&_start_bss;
    while (dst < (uint8_t *)&_end_bss)
      {
	*dst++ = 0;
#ifdef DEBUG_STARTUP
	PA= (uint32_t)dst;
#endif
      }
#endif
    
    
#ifdef DEBUG_STARTUP
    PC= 0x33;
#endif
  main();
 after_main:
    goto after_main;
}

//#include "dev.h"

void Dummy_Handler(void) {
    while (1)
      ;//RGB_RED->led= 1;
}

