/*
 * utime.c
 *
 * (C) Drtos Dniel
 *
 */

#include <stdio.h>
#include <time.h>
#include <sys/time.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>

//#include "defs.h"


/********************************************************************* Time */

time_t
dd_timegm(struct tm *tm)
{
  time_t ret;

#ifdef TARGET_host
  char *tz;
  tz= getenv("TZ");
  setenv("TZ", "", 1);
  tzset();
  ret= mktime(tm);
  if (tz)
    setenv("TZ", tz, 1);
  else
    unsetenv("TZ");
  tzset();
#else
  ret= mktime(tm);
#endif
  return ret;
}

double
dnow(void)
{
  struct timeval tv;
  gettimeofday(&tv, NULL);
  return (double)tv.tv_sec + ((double)tv.tv_usec/1000000.0);
}

double
tv_val(struct timeval *tv)
{
  if (tv == NULL)
    return 0;
  return (double)tv->tv_sec + ((double)tv->tv_usec/1000000.0);
}

static double value_of_tv;

double *
tv_value(struct timeval *tv)
{
  value_of_tv= tv_val(tv);
  return &value_of_tv;
}

int
year(double when)
{
  time_t now;
  struct tm tm;
  if (when<0)
    now= time(NULL);
  else
    now= when;
  gmtime_r(&now, &tm);
  return 1900+tm.tm_year;
}


int
doy(long int when)
{
  time_t now;
  struct tm tm;
  if (when<0)
    {
      now= time(NULL);
    }
  else
    {
      now= when;
    }
  gmtime_r(&now, &tm);
  return tm.tm_yday+1;
}

int
hod(time_t when)
{
  time_t now;
  struct tm tm;
  if (when<0)
    now= time(NULL);
  else
    now= when;
  gmtime_r(&now, &tm);
  return tm.tm_hour;
}

int
sod(time_t when)
{
  time_t now;
  struct tm tm;
  if (when<0)
    now= time(NULL);
  else
    now= when;
  gmtime_r(&now, &tm);
  return tm.tm_hour*60*60+tm.tm_min*60+tm.tm_sec;
}


long int
UTC_of_first_noon(int year)
{
  struct tm tm;
  time_t t;

  t= time(NULL);
  gmtime_r(&t, &tm);
  tm.tm_mon= 0;
  tm.tm_mday= 1;
  tm.tm_hour= 12;
  tm.tm_min= 0;
  tm.tm_sec= 0;
#ifdef HAVE_TM_ZONE
  tm.tm_gmtoff= 0;
  tm.tm_zone= "UTC";
#endif
  if (year>0)
    tm.tm_year= year-1900;
  t= dd_timegm(&tm);
  return t;
}

double
date2utc(int year, int month, int day)
{
  struct tm tm;
  time_t t;
  tm.tm_mon= month-1;
  tm.tm_mday= day;
  tm.tm_hour= 0;
  tm.tm_min= 0;
  tm.tm_sec= 0;
#ifdef HAVE_TM_ZONE
  tm.tm_gmtoff= 0;
  tm.tm_zone= "UTC";
#endif
  if (year>0)
    tm.tm_year= year-1900;
  t= dd_timegm(&tm);
  return t;
}


long int
day_lstart(double when)
{
  struct tm tm;
  time_t t;

  if (when < 0)
    t= time(NULL);
  else
    t= when;
  gmtime_r(&t, &tm);
  tm.tm_hour= 0;
  tm.tm_min= 0;
  tm.tm_sec= 0;
#ifdef HAVE_TM_ZONE
  tm.tm_gmtoff= 0;
  tm.tm_zone= "UTC";
#endif
  t= dd_timegm(&tm);
  return t;
}


double
day_start(double when)
{
  return day_lstart(when);
}


double
utc2julian(double utc)
{
  return utc/86400 + 2440588;
}

double
julian2utc(double julian)
{
  return (julian-2440587.5)*86400.0;
}

void
print_utc(int fd, double utc)
{
  struct tm tm;
  time_t t= utc;
  char s[22];
  int n;

  gmtime_r(&t, &tm);
  n= snprintf(s, 21, "%04d-%02d-%02d %02d:%02d:%02d",
	      tm.tm_year+1900,
	      tm.tm_mon+1,
	      tm.tm_mday,
	      tm.tm_hour,
	      tm.tm_min,
	      tm.tm_sec);
  fflush(NULL);
  write(fd, s, n);
}

char *
format_utc(double utc, char *buf, int size)
{
  struct tm tm;
  time_t t= utc;
  //int n;
  char *s;
  static char format_utc_buf[23];
  
  if (buf==NULL)
    {
      s= format_utc_buf;
      size= 22;
    }
  else
    s= buf;
  
  gmtime_r(&t, &tm);
  /*n=*/ snprintf(s, size, "%04d-%02d-%02d %02d:%02d:%02d",
	      tm.tm_year+1900,
	      tm.tm_mon+1,
	      tm.tm_mday,
	      tm.tm_hour,
	      tm.tm_min,
	      tm.tm_sec);
  return s;
}


double
set_dnow(double d)
{
  struct timeval tp;
  tp.tv_sec= d;
  tp.tv_usec= 0;
  settimeofday(&tp, NULL);
  return dnow();
}


/* End of utime.c */
